@@ -24,7 +24,10 @@ has process_table  => sub { {} };
 has 'handler';
 
 my $singleton;
-sub new { $singleton ||= shift->SUPER::new(@_); }
+
+sub new {
+  $singleton ||= $_[0] ? shift->SUPER::new(@_) : __PACKAGE__->SUPER::new;
+}
 
 sub disable {
   $singleton->_protect(sub { $SIG{CHLD} = $singleton->handler() });
@@ -91,12 +94,12 @@ sub _resolve {
   my ($el, $w) = (pop, pop);
   return
     exists $singleton->{$w}->{$el} ?
-    $el eq 'orphan'
+    $w eq 'orphans'
       ? $singleton->{$w}->{$el}
       : ${$singleton->{$w}->{$el}}
     : undef;
 }
-sub orphan  { _resolve(orphan        => pop()) }
+sub orphan  { _resolve(orphans       => pop()) }
 sub resolve { _resolve(process_table => pop()) }
 
 sub clean {
